/* 
** NetXMS - Network Management System
** NetXMS Foundation Library
** Copyright (C) 2003-2013 Victor Kirhenshtein
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU Lesser General Public License as published
** by the Free Software Foundation; either version 3 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU Lesser General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
** File: strmap.cpp
**
**/

#include "libnetxms.h"

/**
 * Copy constructor
 */
StringMap::StringMap(const StringMap &src) : StringMapBase(true)
{
	m_size = src.m_size;
	m_objectOwner = src.m_objectOwner;
	m_keys = (TCHAR **)malloc(sizeof(TCHAR *) * m_size);
	m_values = (void **)malloc(sizeof(void *) * m_size);
	for(int i = 0; i < m_size; i++)
	{
		m_keys[i] = _tcsdup(src.m_keys[i]);
		m_values[i] = _tcsdup((TCHAR *)src.m_values[i]);
	}
}

/**
 * Destructor
 */

StringMap::~StringMap()
{
}

/**
 * Assignment
 */
StringMap& StringMap::operator =(const StringMap &src)
{
	clear();
	m_size = src.m_size;
	m_keys = (TCHAR **)malloc(sizeof(TCHAR *) * m_size);
	m_values = (void **)malloc(sizeof(void *) * m_size);
	for(int i = 0; i < m_size; i++)
	{
		m_keys[i] = _tcsdup(src.m_keys[i]);
		m_values[i] = _tcsdup((TCHAR *)src.m_values[i]);
	}
	return *this;
}

/**
 * Add all values from another string map
 */
void StringMap::addAll(StringMap *src)
{
   for(int i = 0; i < src->m_size; i++)
      set(src->m_keys[i], (TCHAR *)src->m_values[i]);
}

/**
 * Set value from UINT32
 */
void StringMap::set(const TCHAR *key, UINT32 value)
{
	TCHAR buffer[32];

	_sntprintf(buffer, 32, _T("%u"), (unsigned int)value);
	set(key, buffer);
}

/**
 * Get value by key as UINT32
 */
UINT32 StringMap::getULong(const TCHAR *key, UINT32 defaultValue)
{
	const TCHAR *value = get(key);
	if (value == NULL)
		return defaultValue;
	return _tcstoul(value, NULL, 0);
}

/**
 * Get value by key as boolean
 */
bool StringMap::getBoolean(const TCHAR *key, bool defaultValue)
{
	const TCHAR *value = get(key);
	if (value == NULL)
		return defaultValue;
	if (!_tcsicmp(value, _T("false")))
		return false;
	if (!_tcsicmp(value, _T("true")))
		return true;
	return (_tcstoul(value, NULL, 0) != 0) ? true : false;
}
